# -*- coding: utf-8 -*-
"""Provide the error handling class """


class NvdbError(Exception):
    """Base class for exceptions in this module."""
    pass

class ApiError(NvdbError):
    def __init__(self, message):
        self.message = message

class AutoUpdateError(NvdbError):
    def __init__(self, message):
        self.message = message

def read_api_error(resp):
    try:
        return resp.json()[0]['message']
    except:
        resp.raise_for_status()



# -*- coding: utf-8 -*-
""" Provide  the ObjektType class """
import json
import logging

from .util import _fetch_data, build_name2id


class ObjektType(object):
    """ Class for individual nvdb-object types. (Data catalogue) """

    def __init__(self, nvdb, objekt_type, meta=None):
        self.nvdb = nvdb
        if isinstance(objekt_type, int):
            self.objekt_type = int(objekt_type)
        else:
            if isinstance(self.nvdb.name2id, dict):
                self.objekt_type = self.nvdb.name2id['nvdb_objekter'][objekt_type.lower()]
            else:
                build_name2id(self.nvdb)
                try:
                    self.objekt_type = self.nvdb.name2id['nvdb_objekter'][objekt_type.lower()]
                except KeyError:
                    logging.error('Objekt_type not found: {}'.format(objekt_type))
                    return None
        
        self.data = None
        self.metadata
        logging.debug("Initialized: ObjektType({})".format(self.objekt_type))

    def __repr__(self):
        return "ObjektType({})".format(self.objekt_type)
    def _update_data(self):
        self.data = _fetch_data(
            self.nvdb, 'vegobjekttyper/{}'.format(self.objekt_type))


    def dump(self, file_format='json'):
        """
        Function for dumping raw API-result for object.

        :param file_format: Type of data to dump as. json or xml
        :type file_format: string
        :returns: str
        """
        if file_format.lower() == 'json':
            if not self.data:
                self.data = _fetch_data(self.nvdb, 'vegobjekttyper/{}'
                                        .format(self.objekt_type))
            return self.data

        elif file_format.lower() == 'xml':
            xml_data = _fetch_data(self.nvdb, 'vegobjekttyper/{}.xml'
                                   .format(self.objekt_type), file_format='xml')
            return xml_data

    @property
    def relasjonstyper(self):
        """
        :Attribute type: Dict
        :keys: ['barn', 'foreldre']
        :keys in keys: ['type', 'relasjonstype', 'id']

        """
        if not self.data:
            self._update_data()

        return self.data['relasjonstyper']

    def egenskapstype(self, egenskapstype_id=None):
        """
        Function for returning egenskap based on id

        :param egenskaps_id: Id of the property type you want returned
        :type egenskaps_id: int
        :returns: dict unless property is not found. Then None is returned.
        """
        egenskapstype = list(
            filter(lambda x: x['id'] == egenskapstype_id, self.egenskapstyper))
        if len(egenskapstype):
            return egenskapstype[0]
        return None

    @property
    def egenskapstyper(self):
        """
        :Attribute type: list of Dicts
        :keys: ['liste', 'navn', 'datatype_tekst', 'veiledning', 'beskrivelse', 'sensitivitet',
                'sosinvdbnavn', 'objektliste_dato', 'feltlengde', 'sorteringsnummer', 'id',
                'styringsparametere', 'viktighet', 'viktighet_tekst', 'datatype']
        """
        if not self.data:
            self._update_data()
            
        return self.data['egenskapstyper']

    @property
    def styringsparametere(self):
        """
        :Attribute type: Dict
        :keys: ['abstrakt_type', 'sideposisjon_relevant', 'retning_relevant', 'ajourhold_splitt',
                'må_ha_mor', 'avledet', 'sektype_20k', 'er_dataserie', 'høyde_relevant', 
                'dekningsgrad', 'overlapp, 'filtrering', 'flyttbar', 'tidsrom_relevant', 
                'ajourhold_i', 'kjørefelt_relevant']
        """
        if not self.data:
            self._update_data()
            
        return self.data['styringsparametere']

    @property
    def metadata(self):
        """
        .. todo:: Possible bug. Returns None after reading other attributes

        :Attribute type: Dict
        :keys: ['navn', 'veiledning', 'beskrivelse', 'objektliste_dato', 'sosinvdbnavn', 
                'sorteringsnummer', 'stedfesting', 'id', 'kategorier']
        """
        #if self.meta:
        #    return self.meta
        if not self.data:
            self._update_data()
            
            metadata = self.data.copy()
            del metadata['egenskapstyper']
            del metadata['relasjonstyper']
            del metadata['styringsparametere']
            self.meta = metadata
        return self.meta

    @property
    def barn(self):
        """
        :Attribute type: list of :class:`.ObjektType`
        """
        if not self.data:
            self._update_data()
            
        realasjoner = self.data['relasjonstyper']
        return [ObjektType(self.nvdb, i['type']['id']) for i in realasjoner['barn']]

    @property
    def foreldre(self):
        """
        :Attribute type: list of :class:`.ObjektType`
        """
        if not self.data:
            self._update_data()
            
        realasjoner = self.data['relasjonstyper']
        return [ObjektType(self.nvdb, i['type']['id']) for i in realasjoner['foreldre']]

    def i_objekt_lista(self):
        """
        Function checking of an object type is part of "Objektlista"

        :returns: bool
        """
        if not self.data:
            self._update_data()
            
        if 'objektliste_dato' in self.data:
            return True
        else:
            return False



# -*- coding: utf-8 -*-
""" Contains various helper functions """
import json
import logging

import requests

from .. import config
from .pnvdb_exceptions import ApiError, read_api_error


def _fetch_data(nvdb, url_add, payload=None, file_format='json'):
    """
    function that collects data from the api
    
    :param nvdb: Instance off Nvdb
    :type nvdb: :class:`.Nvdb`
    :param url_add: The API endpoint to fetch
    :type url_add: string
    :param payload: arguments for the api endpoint
    :type payload: string
    :param file_format: one of two values 'json or xml' will return data in this format
    :type file_format: string

    :returns: data as a dictionary
    """
    base_url = config.lesapi_base_url
    if nvdb:
        headers = nvdb.headers
    else:
        headers = None
    url = '{base_url}/{url_add}'.format(base_url=base_url, url_add=url_add)
    resp = requests.get(url, params=payload, headers=headers)

    logging.debug(resp.headers)
    logging.debug(resp.url)
    data = _check_response(resp, file_format)
    return data


def _check_response(resp, file_format='json'):
    """Function verifes that a 200 code was returned from the API
    and returns the data as Json.
    If a 200 code was not returned, it tries to return the error recived 
    from the API."""

    if resp.status_code == 200 and file_format == 'json':
        return resp.json()
    elif resp.status_code == 200 and file_format == 'xml':
        return resp.text
    else:
        raise ApiError(read_api_error(resp))


def build_name2id(nvdb):
    """
    Function that updates nvdb_name2id
    """
    nvdb.name2id = {}
    name_data = _fetch_data(None, 'vegobjekttyper')
    nvdb_objekter = {}
    for objekt in name_data:
        nvdb_objekter[objekt['navn'].lower()] = int(objekt['id'])
    
    nvdb.name2id['nvdb_objekter'] = nvdb_objekter



# -*- coding: utf-8 -*-
""" Provide the Area class """
from .objekt import Objekt
from .util import _fetch_data


class Area(object):
    """ Class for area objects. """

    def __init__(self, nvdb, area_data):
        super(Area, self).__init__()
        self.nvdb = nvdb
        self.data = area_data

    @property
    def metadata(self):
        """ 
        :Attribute type: Dict

        :keys: ['nummer','navn']
        """
        metadata = self.data.copy()
        if 'kartutsnitt' in self.data:
            del metadata['kartutsnitt']
        if 'senterpunkt' in self.data:
            del metadata['senterpunkt']
        if 'vegobjekt' in self.data:
            del metadata['vegobjekt']
        return metadata

    @property
    def kartutsnitt(self):
        """ 
        :Attribute type: Well Known Text
        """
        if 'kartutsnitt' in self.data:
            return self.data['kartutsnitt']['wkt']
        else:
            return None

    @property
    def senterpunkt(self):
        """
        :Attribute type: Well Known Text
        """
        if 'senterpunkt' in self.data:
            return self.data['senterpunkt']['wkt']
        else:
            return None

    @property
    def objekt(self):
        """
        :Attribute type: :class:`.Objekt` of the Area
        """
        objekttype = self.data['vegobjekt']['type']
        nvdb_id = self.data['vegobjekt']['id']
        return Objekt(self.nvdb, objekttype, nvdb_id)



# -*- coding: utf-8 -*-
import logging

from . import config, models
from .models.util import _fetch_data


class Nvdb(object):
    """ The main class for interfacing with the API.

        :param client: Name of client using the API
        :type client: str
        :param contact: Contact information of user of the API
        :type contact: str
        :param autoupdate: Indicated wether constants should be up to date with latest API-Version. Default value = True
        :type autoupdate: Bool
        :returns:  Nvdb Class

        :usage:

        >>> import pnvdb
        >>> nvdb = pnvdb.Nvdb(client='Your-App-Name', contact='Your-contact-information')

    """

    def __init__(self, client='pnvdb', contact='', autoupdate=True):
        self.base_url = config.lesapi_base_url
        self.headers = {'X-Client': client, 'X-Kontaktperson': contact}
        self.srid = ''
        self.antall = 1000
        self.name2id = None
        """
        status = _fetch_data(self, 'status')
        
        if autoupdate and last_seen_version != float(status['datakatalog']['versjon']):
            try:
                update_CONST()
            except:
                print('Autoupdate of the CONST.py file failed.\nTry initializing with adminstrative privleleges, or set autoupdate = False')

            logging.info('Updated name2id and kommune values from version: {} to version {}'.
                         format(last_seen_version, status['datakatalog']['versjon']))
        """
    def _generator(self, url, _payload, objekt_type, data):
        while True:
            returnert = data['metadata']['returnert']
            if returnert == 0:
                break

            _payload.update({'start': data['metadata']['neste']['start']})
            for obj in enumerate(data['objekter']):
                yield models.Objekt(self, objekt_type, obj[1]['id'], obj)
            data = _fetch_data(self, url, _payload)

    def status(self):
        """ Method for getting information about the current status of the API

            :returns: Dict
            :keys: ['datakatalog', 'datagrunnlag']

            :usage:

            >>> status = nvdb.status()
            >>> print(status['datakatalog']['versjon'])
            2.13
        """
        return _fetch_data(self, 'status')

    def objekt(self, objekt_type, nvdb_id):
        """ Method for creating a spesific nvdb python Objekt

            :param objekt_type: nvdb objekttype id.
            :type objekt_type: int
            :param nvdb_id: the unique nvdb id
            :type nvdb_id: int
            :returns: :class:`.Objekt`

            :usage:

            >>> obj = nvdb.objekt(objekt_type=67, nvdb_id=89204552)
            >>> print(obj.metadata)
            {'versjon': 3, 'type':P {'navn': 'Tunnelløp', 'id': 67}, 'startdato': '2014-01-17',
             'sist_modifisert': '2017-10-23 15:15:50'}

        """
        return models.Objekt(self, objekt_type, nvdb_id)

    def objekt_type(self, objekt_type):
        """ Method for creating a spesific nvdb python

            :param objekt_type: nvdb objekttype id.
            :type objekt_type: int
            :returns: :class:`.ObjektType`

            :usage:

            >>> obj = nvdb.objekt_type(objekt_type=67)
            >>> print(obj.metadata['sosinvdbnavn'])
            Tunnelløp_67
        """
        return models.ObjektType(self, objekt_type)

    def objekt_typer(self):
        """ Returns objekt_type of every avaliable obj type in nvdb

            :returns: List of :class:`.ObjektType`

            :usage:

            >>> obj_types = nvdb.objekt_typer()
            >>> print(obj_types[0].metadata['sosinvdbnavn'])
            Skjerm_3

        """
        data = _fetch_data(self, 'vegobjekttyper')
        objekt_typer = []
        for objekt_type in data:
            objekt_type_id = objekt_type['id']
            objekt_typer.append(models.ObjektType(
                self, objekt_type_id, meta=objekt_type))
        return objekt_typer

    def hent(self, objekt_type, kriterie=None):
        """ Return a generator object that can be itterated over
            to fetch the results of the query.

            :param objekt_type: nvdb objekttype id.
            :type objekt_type: int
            :param payload: filters for the query
            :type payload: dict
            :returns: generator of :class:`.Objekt`

            :usage:

            >>> criteria = {'fylke':'2','egenskap':'1820>=20'}
            >>> bomstasjoner = nvdb.hent(45, kriterie=criteria)
            >>> for bomstasjon in bomstasjoner:
            >>>     print(bomstasjon)
        """
        _payload = dict()
        if kriterie:
            _payload = kriterie.copy()
        _payload.update(
            {'antall': self.antall, 'segmentering': 'false', 'inkluder': 'alle'})

        url = 'vegobjekter/{objekt_type}'.format(objekt_type=objekt_type)
        data = _fetch_data(self, url, payload=_payload)
        if data['metadata']['returnert'] == 0:
            return None
        else:
            return self._generator(url, _payload, objekt_type, data)

    def vegreferanse(self, vegreferanse):
        """ Return vegreferanse object.
            PS : Only support point refferences

            :param vegreferanse: The road refferences to objectify
            :type vegreferanse: string
            :returns: :class:`.Vegreferanse`

            :usage:

            >>> print(nvdb.vegreferanse('1600Ev6hp12m1000'))
        """
        if isinstance(vegreferanse, list):
            return [models.Vegreferanse(self, vegref)
                    for vegref in vegreferanse]
        return models.Vegreferanse(self, vegreferanse)

    def posisjon(self, x_coordinate=None, y_coordinate=None, lat=None, lon=None):
        """Returns a posisjon object for a given location

            :param x: X-coordinate in EUREF89 UTM 33
            :type x: float
            :param y: Y-coordinate in EUREF89 UTM 33
            :type y: float
            :param lat: Lattitude in EUREF89
            :type lat: float
            :param lon: Longitude in EUREF89
            :type lon: float
            :returns: :class:`.Posisjon`

            :usage:

            >>> pos = nvdb.posisjon(x=269815,y=7038165)
            >>> print(pos.vegreferanse)
        """
        if x_coordinate and y_coordinate:
            payload = {'nord': y_coordinate, 'ost': x_coordinate}
        elif lat and lon:
            payload = {'lat': lat, 'lon': lon}

        return models.Posisjon(self, payload)

    def regioner(self):
        """ Returns an Area object for all regions

            :returns: list of :class:`.Area`

            :usage:

            >>> for region in nvdb.regioner():
            >>>     print(region.metadata)
        """
        payload = {'inkluder': 'alle'}
        data = _fetch_data(self, 'omrader/regioner', payload)
        return [models.Area(self, models.Area_data) for models.Area_data in data]

    def fylker(self):
        """ Returns an mArea object for all fylker

            :returns: list of :class:`.Area`

            :usage:

            >>> for region in nvdb.regioner():
            >>>     print(region.metadata)
        """
        payload = {'inkluder': 'alle'}
        data = _fetch_data(self, 'omrader/fylker', payload)
        return [models.Area(self, models.Area_data) for models.Area_data in data]

    def vegavdelinger(self):
        """ Returns an Area object for all vegavdelinger

            :returns: list of :class:`.Area`

            :usage:

            >>> for region in nvdb.regioner():
            >>>     print(region.metadata)
        """
        payload = {'inkluder': 'alle'}
        data = _fetch_data(self, 'omrader/vegavdelinger', payload)
        return [models.Area(self, models.Area_data) for models.Area_data in data]

    def kommuner(self):
        """ Returns an Area object for all kommuner

            :returns: list of :class:`.Area`

            :usage:

            >>> for region in nvdb.regioner():
            >>>     print(region.metadata)
        """
        payload = {'inkluder': 'alle'}
        data = _fetch_data(self, 'omrader/kommuner', payload)
        return [models.Area(self, models.Area_data) for models.Area_data in data]

    def kontraktsomrader(self):
        """ Returns an Area object for all kontraktsomrader

            :returns: list of :class:`.Area`

            :usage:

            >>> for region in nvdb.regioner():
            >>>     print(region.metadata)
        """
        payload = {'inkluder': 'alle'}
        data = _fetch_data(self, 'omrader/kontraktsomrader', payload)
        return [models.Area(self, models.Area_data) for models.Area_data in data]

    def riksvegruter(self):
        """ Returns an Area object for all riksvegruter

            :returns: list of :class:`.Area`

            :usage:

            >>> for region in nvdb.regioner():
            >>>     print(region.metadata)
        """
        payload = {'inkluder': 'alle'}
        data = _fetch_data(self, 'omrader/riksvegruter', payload)
        return [models.Area(self, models.Area_data) for models.Area_data in data]



# -*- coding: utf-8 -*-
""" Provide the Objekt class """
import logging

from .util import _fetch_data, build_name2id
from .vegreferanse import Vegreferanse


class Objekt(object):
    """ Class for individual nvdb-objects. """

    def __init__(self, nvdb, objekt_type, nvdb_id, data=None):
        self.nvdb = nvdb
        if isinstance(objekt_type, int):
            self.objekt_type = int(objekt_type)
        else:
            if isinstance(self.nvdb.name2id, dict):
                self.objekt_type = self.nvdb.name2id['nvdb_objekter'][objekt_type.lower()]
            else:
                build_name2id(self.nvdb)
                try:
                    self.objekt_type = self.nvdb.name2id['nvdb_objekter'][objekt_type.lower()]
                except KeyError:
                    logging.error('Objekt_type not found: {}'.format(objekt_type))
                    return None

        self.nvdb_id = int(nvdb_id)
        if data:
            self.data = data[1]
        else:
            self.data = data
        logging.debug("Initialized: Objekt({}, {})".format(self.objekt_type, self.nvdb_id))

    def __repr__(self):
        return "Objekt({}, {})".format(self.objekt_type, self.nvdb_id)

    def _update_data(self):
        self.data = _fetch_data(self.nvdb, 'vegobjekter/{}/{}'
                        .format(self.objekt_type, self.nvdb_id),
                        payload={'inkludergeometri': 'utledet', 'inkluder':'alle'})
        

    @property
    def egengeometri(self):
        """
        Boolean value that tell if the object has egengeometri or not.

        :returns: Bool. If it's not found it will return None
        """
        if not self.data:
            self._update_data()
        
        if 'egengeometri' in self.data['geometri']:
            egengeometri = self.data['geometri']['egengeometri']
        else:
            egengeometri = None
        
        return egengeometri

    def egenskap(self, egenskaps_id=None):
        """
        Function for returning egenskap based on id

        :param egenskaps_id: Id of the property type you want returned
        :type egenskaps_id: int
        :returns: dict unless property is not found. Then None is returned.
        """
        egenskap = list(
            filter(lambda x: x['id'] == egenskaps_id, self.egenskaper))
        if len(egenskap):
            return egenskap[0]
        return None

    @property
    def egenskaper(self):
        """
        :Attribute type: List of Dict
        :keys: ['datatype_tekst', 'id', 'datatype', 'verdi', 'navn']
        """
        if not self.data:
            self._update_data()

        if 'egenskaper' in self.data:
            egenskaper = self.data['egenskaper']
        else:
            egenskaper = None
        return egenskaper

    @property
    def metadata(self):
        """
        :Attribute type: Dict
        :keys: ['versjon', 'sist_modifisert', 'startdato', 'type']
        """
        if not self.data:
            self._update_data()
            
        if 'metadata' in self.data:
            metadata = self.data['metadata']
        else:
            metadata = None
        return metadata

    @property
    def geometri(self):
        """
        :Attribute type: Well Known Text
        """
        if not self.data:
            self._update_data()
            
        if 'geometri' in self.data:
            geometri = self.data['geometri']['wkt']
        else:
            geometri = None
        return geometri

    def dump(self, file_format='json'):
        """
        Function for dumping raw API-result for object.

        :param file_format: Type of data to dump as. json or xml
        :type file_format: string
        :returns: str
        """
        if file_format.lower() == 'json':
            if not self.data:
                self._update_data()
            
            return self.data
        elif file_format.lower() == 'xml':
            xml_data = _fetch_data(self.nvdb, 'vegobjekter/{}/{}.xml'
                                   .format(self.objekt_type, self.nvdb_id), file_format='xml')
            return xml_data

    @property
    def foreldre(self):
        """
        :Attribute type: List of :class:`.Objekt`
        """
        if not self.data:
            self._update_data()
            
        foreldre = []
        if 'relasjoner' in self.data and 'foreldre' in self.data['relasjoner']:
            for i in self.data['relasjoner']['foreldre']:
                objekt_type = i['type']['id']
                for nvdb_id in i['vegobjekter']:
                    foreldre.append(Objekt(self.nvdb, objekt_type, nvdb_id))
        else:
            foreldre = None
        return foreldre

    

    @property
    def vegreferanser(self):
        """
        :Attribute type: :class:`.Vegreferanse`

        """
        if not self.data:
            self._update_data()
            
        vegreferanser = []
        if 'lokasjon' in self.data and 'vegreferanser' in self.data['lokasjon']:
            for i in self.data['lokasjon']['vegreferanser']:
                vegreferanser.append(i['kortform'])
        else:
            vegreferanser = None
        return vegreferanser

    @property
    def vegsegmenter(self):
        """
        :Attribute type: list of dict
        :keys: []

        """
        if not self.data:
            self._update_data()
            
        return self.data['vegsegmenter']

    @property
    def kommuner(self):
        """
        :Attribute type: list of dict
        :keys: [fylke, navn, nummer, region, vegavdeling]

        """
        if not self.data:
            self._update_data()
            
        from ..const import KOMMUNER
        alle_kommuner = KOMMUNER
        kommuner = []
        for kommune_nr in self.data['lokasjon']['kommuner']:
            kommuner.append(alle_kommuner[str(kommune_nr)])
        return kommuner
    
    @property
    def barn(self):
        """
        :Attribute type: List of :class:`.Objekt`

        """
        if not self.data:
            self._update_data()
            
        barn = []
        if 'relasjoner' in self.data and 'barn' in self.data['relasjoner']:
            for i in self.data['relasjoner']['barn']:
                objekt_type = i['type']['id']
                for nvdb_id in i['vegobjekter']:
                    barn.append(Objekt(self.nvdb, objekt_type, nvdb_id))

        else:
            barn = None
        return barn


import geojson


class Feature(object):
    '''Class for defining objects ready to push to Datafangst'''

    def __init__(self, objekt_type, coordinates, tag):
        self._objekt_type = objekt_type
        self._coordinates = None
        self.properties = {}
        self.coordinates(coordinates)
        self.properties["typeId"] = objekt_type
        self.properties["tag"] = tag
        self.properties["dataCatalogVersion"] = self._current_datakatalog()
        self.properties['attributes'] = {}

    def _current_datakatalog(self):
        """ Returns current datakatalog version """
        from ..les import Nvdb
        status = Nvdb.status(None)
        return status['datakatalog']['versjon']

    def coordinates(self, geometry):
        """ Method for setting the geometry of the feature
            
            :param coordinates: Coordinates describing the feature geometry
            :type coordinates: list of tuples or singel tuple for points
        """
        if isinstance(geometry, list):
            if geometry[0] == geometry[-1]:
                self._coordinates = geojson.Polygon([geometry])
            else:
                self._coordinates = geojson.LineString(geometry)
        else:
            self._coordinates = geojson.Point(geometry)

    def attribute(self, attribute_id, attribute_value):
        """ Method for adding an attribute to the feature
            
            :param attribute_id: nvdb attribute ID
            :type attribute_id: int
            :param attribute_value: value for the attribute
            :type attribute_value: str or int
        """
        self.properties['attributes'][str(attribute_id)] = str(attribute_value)

    def comment(self, comment):
        """ Method for adding an commnet to the feature
            
            :param comment: The comment to add to the feature
            :type comment: str
        """
        self.properties["comment"] = comment

    def tag(self, tag):
        """ Method for adding a tag to the feature
        :param tag: Identifying tag for the feature. Identical tags will be made uniqe with a number.
        :type tag: str
        """
        self.properties["tag"] = tag


    def __repr__(self):
        feature = geojson.Feature(geometry=self._coordinates,
                                  properties=self.properties)
        return geojson.dumps(feature)



# -*- coding: utf-8 -*-
""" Make avaliable the various classes to the API """
from .area import Area
from .objekt import Objekt
from .objekt_type import ObjektType
from .posisjon import Posisjon
from .vegreferanse import Vegreferanse
from .feature import Feature
from .feature_collection import FeatureCollection

__all__ = ('Objekt', 'Vegreferanse', 'Area', 'objekt_type',
           'posisjon', 'feature', 'feature_collection')



# -*- coding: utf-8 -*-
"""
pnvdb
====


pnvdb stands for "python for Nasjonal vegdatabank" and is a python wrapper for the `NVDB Rest API <https://www.vegvesen.no/nvdb/apidokumentasjon/>`_


Data pulled from the API is licensed under
Norsk lisens for offentlige data `NLOD <http://data.norge.no/nlod/no/1.0>`_

Getting started
---------------

Read the docs at 
http://pnvdb.readthedocs.io

Install
^^^^^^^
::

    pip install pnvdb


Quick Start
^^^^^^^^^^^

Start with initalizing an instance of pnvdb::

    >>> import pnvdb
    >>> nvdb = pnvdb.Nvdb(client='Your-App-Name', contact='Your-contact-information')

Now we can test our connection to NVDB::

    >>> print(nvdb.status())
    {'datagrunnlag': {'sist_oppdatert': '2017-11-05 11:59:37'}, 'datakatalog': {'id': 782, 'dato': '2017-09-29', 'versjon': '2.10'}}


To work with a spesific nvdb object, we can initialize it like this::

    >>> objekt = nvdb.objekt(objekt_type=67, nvdb_id=86543444)
    

This will get us access to a number of attributes assosiated with this object. Let's print one out::

    >>> print(objekt.metadata)
    {'type': {'id': 67, 'navn': 'Tunnelløp'}, 'versjon': 14, 'startdato': '2014-09-19', 'sist_modifisert': '2017-10-24 15:40:48'}

We can search using area and property filters.
This will return a generator object that can be itterated over.

    >>> criteria = {'fylke':'2','egenskap':'1820>=20'} # 1820 = "Takst liten bil"

    >>> bomstasjoner = nvdb.hent(45, criteria)
    >>> obj = nvdb.hent(45, criteria)
    >>> for i in obj:
    >>>     for egenskap in i.egenskaper:
    >>>         if egenskap['id'] == 1078: # 1078 = "Navn bomstasjon"
    >>>             print(egenskap['verdi'])
"""
from .les import Nvdb
from .datafangst import Datafangst
__version__ = '0.4.0'



import json

import geojson
import requests


class FeatureCollection(object):
    ''' Class for defining a set of objects ready to push to datafangst'''

    def __init__(self, url, username, password, headers):
        self.username = username
        self.password = password
        self.features = []
        self.status_src = None
        self.url = url
        self.headers = headers

    def add_feature(self, feature):
        """ Method that adds a `.Feature` to the instance"""
        self.features.append(geojson.Feature(geometry=feature._coordinates,
                                             properties=feature.properties))

    def push(self):
        """ Method that pushes the FeatureCollection to datafangst
            
            :returns: xml respons from datafangst
        """
        feature_collection = geojson.FeatureCollection(self.features)
        response = requests.post(self.url, data=geojson.dumps(feature_collection).encode('utf8'),
                                 headers=self.headers, auth=(self.username, self.password))

        if response.status_code == 202:
            self.status_src = json.loads(response.text)['resources'][1]['src']
            return self.status()
        else:
            response.raise_for_status()

    def status(self):
        """ Method for polling the status of the instance from datafangst
            returns None if data not pushed to datafangst
            
            :returns: xml respons from datafangst
        """
        if self.status_src:
            return requests.get(self.status_src, auth=(self.username, self.password)).text
        else:
            return 'Not pushed'
    
    def __repr__(self):
        return self.status




import json

from . import config, models

class Datafangst(object):
    """ Main class for interfacing with the 'Datafangst' API

        :param username: Datafangst username
        :type username: str
        :param password: Datafangst password
        :type password: str
        :param contractId: Datafangst contract ID
        :type contractId: str
        :returns: :class:`.Datafangst`

        :usage:

        >>> import pnvdb
        >>> datafangst = pnvdb.Datafangst(username, password, contractId)

    """
    def __init__(self, username=None, password=None, contractId=None):
        self.base_url = config.datafangst_base_url
        self.username = username
        self.password = password
        self.headers = {
            'Content-Type': "application/geo+json",
            'Accept': "application/json"
        }
        self.url = "{baseurl}/{contractId}/featurecollection".format(
            baseurl=self.base_url, contractId=contractId)

    def feature_collection(self):
        """
        Method for initializing and working with a datafangst feature collection
        
        :returns: :class:`.FeatureCollection`
        """
        return models.FeatureCollection(self.url, self.username, self.password, self.headers)

    def feature(self, objekt_type, coordinates, tag):
        """
        Method for initialzing and working with a datafangst feature
        
        :param objekt_type: NVDB object type of the feature
        :type objekt_type: int
        :param coordinates: Coordinates describing the feature geometry
        :type coordinates: list of tuples or singel tuple for points
        :param tag: Identifying tag for the feature. Identical tags will be made uniqe with a number.
        :type tag: str
        
        :returns: :class:`.Feature`
        """
        return models.Feature(objekt_type, coordinates, tag)



# -*- coding: utf-8 -*-
""" Provide the Vegreferanse class """
from .util import _fetch_data
import logging


class Vegreferanse(object):
    """ Class for working with road refferences.
    """

    def __init__(self, nvdb, vegreferanse):
        self.vegreferanse = vegreferanse
        self.nvdb = nvdb
        
        self.data = None
        logging.debug('Initialized vegreferanse: {}'.format(self.vegreferanse))
        
    def _update_data(self):
        self.data = _fetch_data(self.nvdb, 'veg', payload={
                            'vegreferanse': self.vegreferanse})

    @property
    def fylke(self):
        """
        The county of the road refference
        :Attribute type: int
        """
        if not self.data:
            self._update_data()

        return self.data['vegreferanse']['fylke']
    
    @property
    def kommune(self):
        """
        The kommune of the road refference
        :Attribute type: int
        """
        if not self.data:
            self._update_data()
            
        return self.data['vegreferanse']['kommune']
    
    @property
    def kategori(self):
        """
        The kategori of the road refference
        :Attribute type: String
        """
        if not self.data:
            self._update_data()
            
        return self.data['vegreferanse']['kategori']
    
    @property
    def status(self):
        """
        The status of the road refference
        :Attribute type: String
        """
        if not self.data:
            self._update_data()
            
        return self.data['vegreferanse']['status']
    
    @property
    def nummer(self):
        """
        The nummer of the road refference
        :Attribute type: int
        """
        if not self.data:
            self._update_data()
            
        return self.data['vegreferanse']['nummer']
    
    @property
    def hp(self):
        """
        The hp of the road refference
        :Attribute type: int
        """
        if not self.data:
            self._update_data()
            
        return self.data['vegreferanse']['hp']
    
    @property
    def meter(self):
        """
        The meter of the road refference
        :Attribute type: int
        """
        if not self.data:
            self._update_data()
            
        return self.data['vegreferanse']['meter']
    
    @property
    def geometri(self):
        if not self.data:
            self._update_data()
            
        return self.data['geometri']['wkt']

    @property
    def xyz(self):
        import re
        if not self.data:
            self._update_data()
            
        wkt = self.data['geometri']['wkt']
        reg_res = re.findall(r'\d+\.?\d*', wkt)
        if len(reg_res) < 3:
            x,y = reg_res
            z = None
        else:
            x,y,z = reg_res
        return x,y,z
    
    def __repr__(self):
        return '{}'.format(self.vegreferanse)